@@ -12,7 +12,7 @@ class Config():
             setattr(self, key, kwargs[key])
     
     def save(self, path):
-        pkl.dump(config.__dict__, open(path, "wb"))
+        pkl.dump(self.__dict__, open(path, "wb"))
     
     @property
     def default_run_dir(self):
