@@ -3,6 +3,7 @@ import sys
 
 import pyhectiqlab
 from pyhectiqlab.metrics import MetricsManager
+from pyhectiqlab.settings import app_url
 from pyhectiqlab.events_manager import EventsManager
 from pyhectiqlab.stream import LogHandler
 from pyhectiqlab.buffer import Buffer
@@ -18,6 +19,7 @@ from pyhectiqlab.datasets import upload_dataset as ops_upload_dataset
 from pyhectiqlab.mlmodels import mlmodel_info_at_path
 
 from functools import partial, wraps
+from typing import Optional
 from sys import getsizeof
 import packaging 
 import tempfile
@@ -26,18 +28,21 @@ import shutil
 
 class Run():
     
-    def __init__(self, name:str, project:str=None, project_name:str=None, should_create_project = False, dry=False):
+    def __init__(self, name: str, project: str = None, should_create_project: bool = False, dry: bool = False):
         """
-        dry: bool: If true, the run does not send anything.
-        should_create_project: bool. If true, a project is created if if does not exists
+        name: Run name. If a run exists with this name, you'll be attached to the existing run. Otherwise a 
+            new run is created.
+        project: The project name. Leave None if the user with the API key has access to a single project. 
+            Otherwise, you must specify the project name.
+        should_create_project: bool. If true, creates a project d if if does not exists (Admin only).
+        dry: bool: If true, the run is in a dry mode (no writting).
+        
         """
         self._mock = dry
         self.events_manager = EventsManager(self._mock)
         # Check the python version
-        self.events_manager.compare_python_version()    
-        if project_name is not None:
-            print('Argument `project_name` will be remove in a future release. Please use `project` instead.')
-            project = project_name
+        self.events_manager.compare_python_version()  
+
         if dry:
             print("Dry mode activated.")
             self.run_view = {"id": "null"}
@@ -106,9 +111,21 @@ class Run():
             return f(self, *args, **kwargs)
         return wrapper
 
-    @property
-    def read_only(self):
-        return self._read_only
+    def beta(f):
+        @wraps(f)
+        def wrapper(self, *args, **kwargs):
+            print('This method is still in development and can be unstable.')
+            return f(self, *args, **kwargs)
+        return wrapper
+
+    def will_be_depreciated(suggested_method):
+        def _method(f):
+            @wraps(f)
+            def wrapper(self, *args, **kwargs):
+                print(f'This method will be removed in a future release. Use {suggested_method} instead.')
+                return f(self, *args, **kwargs)
+            return wrapper
+        return _method
 
     def set_metrics_cache_settings(self, aggr: str = 'mean', max_cache_timeout=None, max_cache_length=None, min_cache_flush_delay=None):
         """
@@ -137,6 +154,7 @@ class Run():
         """Log a file as an artifacts. If a file already exists with the name, it will be overwritten.
 
         filepath [str] : Path to the file.
+        step [int]: The optional step stamp of the artifacts.
         """
         filename = os.path.basename(filepath)
         num_bytes = os.path.getsize(filepath)
@@ -145,11 +163,15 @@ class Run():
         self.events_manager.add_event("add_file", args)
 
     @write_method
+    @beta
     def add_mlmodel_usage(self, mlmodel_name: str, version: str = None):
+        """
+        """
         args = (self._id, mlmodel_name, version, self.project_view['id'])
         self.events_manager.add_event("log_mlmodel", args)
 
     @write_method
+    @beta
     def add_mlmodel_usage_from_dirpath(self, dirpath:str):
         """Log that this run is using a certain model. 
         """
@@ -163,8 +185,16 @@ class Run():
         
     @write_method        
     def add_mlmodel(self, source_path: str, name: str, version: str = None, short_description: str = None,
-                push_dir: bool = False, show_filenames: bool = False):
-        """Add a mlmodel
+                push_dir: bool = False):
+        """Add a mlmodel.
+
+        source_path: Path to the directory/file of the mlmodel
+        name: Name of the model without spaces and backslash (e.g., 'text-model')
+        version: Version in format '{major}.{minor}.{micro}' (e.g., 1.2.0). If None, 
+            the version 1.0.0 is assigned or an increment of minor of the latest version of the
+            model with this name (e.g. 1.3.3 -> 1.4.0)
+        short_description: Short description.
+        push_dir: Must be set to True if the source_path is a directory
         """
         ops_upload_mlmodel(source_path=source_path, 
             mlmodel_name=name,
@@ -174,7 +204,14 @@ class Run():
              push_dir=push_dir)
 
     def download_mlmodel(self, mlmodel_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
+        """Download an existing mlmodel from the run's project.
 
+        source_path: Path to the directory/file of the mlmodel
+        mlmodel_name: Name of the model without spaces and backslash (e.g., 'text-model')
+        version: Specific version of the model. If None, the latest version is fetched.
+        save_path: Path to where the model is saved.
+        overwrite: Set to True to overwrite in save_path.
+        """
         dirpath = ops_download_mlmodel(mlmodel_name=mlmodel_name, 
                         project_id=self.project_view["id"], 
                         version=version, 
@@ -186,8 +223,16 @@ class Run():
         
     @write_method
     def add_dataset(self, source_path: str, name: str, version: str = None, short_description: str = None,
-                push_dir: bool = False, show_filenames: bool = False):
-        """Add a mlmodel
+                push_dir: bool = False):
+        """Add a dataset.
+
+        source_path: Path to the directory/file of the dataset
+        name: Name of the dataset without spaces and backslash (e.g., 'text-dataset')
+        version: Version in format '{major}.{minor}.{micro}' (e.g., 1.2.0). If None, 
+            the version 1.0.0 is assigned or an increment of minor of the latest version of the
+            dataset with this name (e.g. 1.3.3 -> 1.4.0)
+        short_description: Short description.
+        push_dir: Must be set to True if the source_path is a directory
         """
         ops_upload_dataset(source_path=source_path, 
             dataset_name=name,
@@ -197,7 +242,14 @@ class Run():
              push_dir=push_dir)
 
     def download_dataset(self, dataset_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
+        """Download an existing dataset from the run's project.
 
+        source_path: Path to the directory/file of the dataset
+        mlmodel_name: Name of the dataset without spaces and backslash (e.g., 'text-dataset')
+        version: Specific version of the dataset. If None, the latest version is fetched.
+        save_path: Path to where the dataset is saved.
+        overwrite: Set to True to overwrite in save_path.
+        """
         dirpath = ops_download_dataset(dataset_name=dataset_name, 
                         project_id=self.project_view["id"], 
                         version=version, 
@@ -208,11 +260,23 @@ class Run():
         return dirpath
     
     @write_method
-    def add_config(self, config, prefix=""):
-        config.push_as_meta(self, prefix=os.path.join("config/",prefix))
+    def add_config(self, config: 'Config', prefix=""):
+        """Log a config object (pyhectiqlab.Config). Use prefix 
+        to log a config in a nested path.
+        """
+        config.push_as_meta(self, prefix=os.path.join("config/", prefix))
 
     @write_method
-    def add_tf_model(self, model, filename, step: int = None):
+    @will_be_depreciated('add_tf_model_as_artifact')
+    def add_tf_model(self, model: 'tf.keras.Model', filename: 'str', step: int = None):
+        """Add a tensorflow model as an artifact.
+        """
+        self.add_tf_model_as_artifact(model=model, filename=filename, step=step)
+
+    @write_method
+    def add_tf_model_as_artifact(self, model: 'tf.keras.Model', filename: str, step: int = None):
+        """Add a tensorflow model as an artifact.
+        """
         save_path = tempfile.mkdtemp()
         model.save_weights(f"{save_path}/{filename}")
         p = tempfile.mkdtemp()
@@ -220,14 +284,14 @@ class Run():
         self.add_artifacts(f"{p}/{filename}.zip", step=step)
 
     @write_method
-    def add_directory_as_zip_artifact(self, dirpath, step: int = None):
+    def add_directory_as_zip_artifact(self, dirpath: str, step: int = None):
         p = tempfile.mkdtemp()
         filename = os.path.basename(dirpath)
         shutil.make_archive(f"{p}/{filename}", 'zip', root_dir=dirpath)
         self.add_artifacts(f"{p}/{filename}.zip", step=step)
 
     @write_method
-    def add_shared_artifact(self, filename):
+    def add_shared_artifact(self, filename: str):
         """Push an artifact to be shared accross the project.
         """
         m = SharedArtifactsManager()
@@ -240,14 +304,13 @@ class Run():
         return m.list_artifacts(project_id=self.project_view["id"])
 
     @write_method
-    def add_metrics(self, key, value, step):
-        """Add a metrics. The metrics is appended even if there already exists a value
+    def add_metrics(self, key: 'str', value: float, step: int):
+        """Add a metrics. The metrics is saved even if there already exists a value
         for the same key and step. 
 
         key [str]: Key used for the metrics. The format with backlash such as `metrics/train`
-            is recognized to organize the metrics.
-
-        value [float or string]. If tensor, is casted to float.
+            will be recognized to organize the metrics.
+        value [float]. If tensor, is casted to float.
         step [int]: Training step. Casted to integer.
         """
         self.metrics_manager.add(key, float(value), int(step))
@@ -261,7 +324,7 @@ class Run():
         self.events_manager.add_event("add_tag", args)
 
     @write_method
-    def add_package_versions(self, packages=None, with_sys=True, with_python=True, with_git=True):
+    def add_package_versions(self, packages: Optional[dict] =None, with_sys: bool = True, with_python: bool = True, with_git: bool = True):
         """Save the version of the imported packages.
 
         Use:
@@ -300,7 +363,7 @@ class Run():
     @write_method
     def add_current_notebook(self, stamp: str = 'datetime'):
         """If the run is executed in a jupyter notebook, this
-        will quickly add your current notebook as an artifact.
+        will add your current notebook as an artifact.
 
         stamp [str]: Add a stamp on the notebook name.
                     One of 'datetime', 'date', None
@@ -432,4 +495,40 @@ class Run():
     @property
     def _id(self):
         return self.run_view["id"]
+
+    @property
+    def read_only(self):
+        return self._read_only
+
+    @property
+    def dry_mode(self):
+        return self._mock
+
+    @property
+    def action_mode(self):
+        if self.dry_mode:
+            return 'dry'
+        if self.read_only:
+            return 'read-only'
+        return 'read-write'
+    
+    @property
+    def author(self):
+        author = self.run_view.get('author')
+        if author is None:
+            return 'Undefined'
+        return author.get('firstname') + ' ' + author.get('lastname')
+        
+    def __str__(self):
+        # print(self.run_view)
+        pad = 9
+        path = f"/{self.project_view['id']}/runs/{self._id}"
+        return f"<Run {self.run_view['p_id']}>"\
+                f"\n{'project'.ljust(pad)}: {self.project_view['name']}"\
+                f"\n{'author'.ljust(pad)}: {self.author}"\
+                f"\n{'mode'.ljust(pad)}: {self.action_mode}"\
+                f"\n{'url'.ljust(pad)}: {app_url+path} "
+
+    def __repr__(self):
+        return self.__str__()
     
\ No newline at end of file
