@@ -3,7 +3,7 @@ from pyhectiqlab import Run
 
 class LabCallback(tf.keras.callbacks.Callback):
 
-    def __init__(self, run: Run):
+    def __init__(self, run: Run, postfix: str):
         self._run = run
         self.epoch = 0
         self.global_step = 0
